import {getToken} from '@/utils/auth'

/**
 * Created by PanJiaChen on 16/11/18.
 */

/**
 * Parse the time to string
 * @param {(Object|string|number)} time
 * @param {string} cFormat
 * @returns {string | null}
 */
export function parseTime(time, cFormat) {
    if (arguments.length === 0 || !time) {
        return null
    }
    const format = cFormat || '{y}-{m}-{d} {h}:{i}:{s}'
    let date
    if (typeof time === 'object') {
        date = time
    } else {
        if (typeof time === 'string') {
            if (/^[0-9]+$/.test(time)) {
                // support "1548221490638"
                time = parseInt(time)
            } else {
                // support safari
                // https://stackoverflow.com/questions/4310953/invalid-date-in-safari
                time = time.replace(new RegExp(/-/gm), '/')
            }
        }

        if (typeof time === 'number' && time.toString().length === 10) {
            time = time * 1000
        }
        date = new Date(time)
    }
    const formatObj = {
        y: date.getFullYear(),
        m: date.getMonth() + 1,
        d: date.getDate(),
        h: date.getHours(),
        i: date.getMinutes(),
        s: date.getSeconds(),
        a: date.getDay()
    }
    const time_str = format.replace(/{([ymdhisa])+}/g, (result, key) => {
        const value = formatObj[key]
        // Note: getDay() returns 0 on Sunday
        if (key === 'a') {
            return ['日', '一', '二', '三', '四', '五', '六'][value]
        }
        return value.toString().padStart(2, '0')
    })
    return time_str
}

/**
 * @param {number} time
 * @param {string} option
 * @returns {string}
 */
export function formatTime(time, option) {
    if (('' + time).length === 10) {
        time = parseInt(time) * 1000
    } else {
        time = +time
    }
    const d = new Date(time)
    const now = Date.now()

    const diff = (now - d) / 1000

    if (diff < 30) {
        return '刚刚'
    } else if (diff < 3600) {
        // less 1 hour
        return Math.ceil(diff / 60) + '分钟前'
    } else if (diff < 3600 * 24) {
        return Math.ceil(diff / 3600) + '小时前'
    } else if (diff < 3600 * 24 * 2) {
        return '1天前'
    }
    if (option) {
        return parseTime(time, option)
    } else {
        return (
            d.getMonth() +
            1 +
            '月' +
            d.getDate() +
            '日' +
            d.getHours() +
            '时' +
            d.getMinutes() +
            '分'
        )
    }
}

/**
 * @param {string} url
 * @returns {Object}
 */
export function getQueryObject(url) {
    url = url == null ? window.location.href : url
    const search = url.substring(url.lastIndexOf('?') + 1)
    const obj = {}
    const reg = /([^?&=]+)=([^?&=]*)/g
    search.replace(reg, (rs, $1, $2) => {
        const name = decodeURIComponent($1)
        let val = decodeURIComponent($2)
        val = String(val)
        obj[name] = val
        return rs
    })
    return obj
}

/**
 * @param {string} input value
 * @returns {number} output value
 */
export function byteLength(str) {
    // returns the byte length of an utf8 string
    let s = str.length
    for (var i = str.length - 1; i >= 0; i--) {
        const code = str.charCodeAt(i)
        if (code > 0x7f && code <= 0x7ff) s++
        else if (code > 0x7ff && code <= 0xffff) s += 2
        if (code >= 0xdc00 && code <= 0xdfff) i--
    }
    return s
}

/**
 * @param {Array} actual
 * @returns {Array}
 */
export function cleanArray(actual) {
    const newArray = []
    for (let i = 0; i < actual.length; i++) {
        if (actual[i]) {
            newArray.push(actual[i])
        }
    }
    return newArray
}

/**
 * @param {Object} json
 * @returns {Array}
 */
export function param(json) {
    if (!json) return ''
    return cleanArray(
        Object.keys(json).map(key => {
            if (json[key] === undefined) return ''
            return encodeURIComponent(key) + '=' + encodeURIComponent(json[key])
        })
    ).join('&')
}

/**
 * @param {string} url
 * @returns {Object}
 */
export function param2Obj(url) {
    const search = decodeURIComponent(url.split('?')[1]).replace(/\+/g, ' ')
    if (!search) {
        return {}
    }
    const obj = {}
    const searchArr = search.split('&')
    searchArr.forEach(v => {
        const index = v.indexOf('=')
        if (index !== -1) {
            const name = v.substring(0, index)
            const val = v.substring(index + 1, v.length)
            obj[name] = val
        }
    })
    return obj
}

/**
 * @param {string} val
 * @returns {string}
 */
export function html2Text(val) {
    const div = document.createElement('div')
    div.innerHTML = val
    return div.textContent || div.innerText
}

/**
 * Merges two objects, giving the last one precedence
 * @param {Object} target
 * @param {(Object|Array)} source
 * @returns {Object}
 */
export function objectMerge(target, source) {
    if (typeof target !== 'object') {
        target = {}
    }
    if (Array.isArray(source)) {
        return source.slice()
    }
    Object.keys(source).forEach(property => {
        const sourceProperty = source[property]
        if (typeof sourceProperty === 'object') {
            target[property] = objectMerge(target[property], sourceProperty)
        } else {
            target[property] = sourceProperty
        }
    })
    return target
}

/**
 * @param {HTMLElement} element
 * @param {string} className
 */
export function toggleClass(element, className) {
    if (!element || !className) {
        return
    }
    let classString = element.className
    const nameIndex = classString.indexOf(className)
    if (nameIndex === -1) {
        classString += ' ' + className
    } else {
        classString =
            classString.substr(0, nameIndex) +
            classString.substr(nameIndex + className.length)
    }
    element.className = classString
}

/**
 * 处理拖拽属性的添加与删除
 * @param node
 * @param isAdd
 */
export function handleDraggable(name, isAdd) {
    let allnode = document.querySelectorAll('span.draggable');
    let selectNode = null;
    for (let i = 0; i < allnode.length; i++) {
        if (allnode[i].innerHTML.indexOf(name) != -1) {
            selectNode = allnode[i]
        }
    }
    if (isAdd) {
        selectNode.setAttribute('draggable', true)
    } else {
        selectNode.removeAttribute('draggable')
    }
}

/**
 * @param {string} type
 * @returns {Date}
 */
export function getTime(type) {
    if (type === 'start') {
        return new Date().getTime() - 3600 * 1000 * 24 * 90
    } else {
        return new Date(new Date().toDateString())
    }
}

/**
 * @param {Function} func
 * @param {number} wait
 * @param {boolean} immediate
 * @return {*}
 */
export function debounce(func, wait, immediate) {
    let timeout, args, context, timestamp, result

    const later = function () {
        // 据上一次触发时间间隔
        const last = +new Date() - timestamp

        // 上次被包装函数被调用时间间隔 last 小于设定时间间隔 wait
        if (last < wait && last > 0) {
            timeout = setTimeout(later, wait - last)
        } else {
            timeout = null
            // 如果设定为immediate===true，因为开始边界已经调用过了此处无需调用
            if (!immediate) {
                result = func.apply(context, args)
                if (!timeout) context = args = null
            }
        }
    }

    return function (...args) {
        context = this
        timestamp = +new Date()
        const callNow = immediate && !timeout
        // 如果延时不存在，重新设定延时
        if (!timeout) timeout = setTimeout(later, wait)
        if (callNow) {
            result = func.apply(context, args)
            context = args = null
        }

        return result
    }
}

/**
 * This is just a simple version of deep copy
 * Has a lot of edge cases bug
 * If you want to use a perfect deep copy, use lodash's _.cloneDeep
 * @param {Object} source
 * @returns {Object}
 */
export function deepClone(source) {
    if (!source && typeof source !== 'object') {
        throw new Error('error arguments', 'deepClone')
    }
    const targetObj = source.constructor === Array ? [] : {}
    Object.keys(source).forEach(keys => {
        if (source[keys] && typeof source[keys] === 'object') {
            targetObj[keys] = deepClone(source[keys])
        } else {
            targetObj[keys] = source[keys]
        }
    })
    return targetObj
}

/**
 * @param {Array} arr
 * @returns {Array}
 */
export function uniqueArr(arr) {
    return Array.from(new Set(arr))
}

/**
 * @returns {string}
 */
export function createUniqueString() {
    const timestamp = +new Date() + ''
    const randomNum = parseInt((1 + Math.random()) * 65536) + ''
    return (+(randomNum + timestamp)).toString(32)
}

/**
 * Check if an element has a class
 * @param {HTMLElement} elm
 * @param {string} cls
 * @returns {boolean}
 */
export function hasClass(ele, cls) {
    return !!ele.className.match(new RegExp('(\\s|^)' + cls + '(\\s|$)'))
}

/**
 * Add class to element
 * @param {HTMLElement} elm
 * @param {string} cls
 */
export function addClass(ele, cls) {
    if (!hasClass(ele, cls)) ele.className += ' ' + cls
}

/**
 * Remove class from element
 * @param {HTMLElement} elm
 * @param {string} cls
 */
export function removeClass(ele, cls) {
    if (hasClass(ele, cls)) {
        const reg = new RegExp('(\\s|^)' + cls + '(\\s|$)')
        ele.className = ele.className.replace(reg, ' ')
    }
}

/**
 * 获取浏览器可视区域宽度
 */
export function getViewPortOffset() {
    if (window.innerWidth) {
        return {
            w: window.innerWidth,
            h: window.innerHeight
        }
    } else if ((document.compatMode = 'BackCompat')) {
        return {
            w: document.body.clientWidth,
            h: document.body.clientHeight
        }
    } else {
        return {
            w: document.documentElement.clientWidth,
            h: document.documentElement.clientHeight
        }
    }
}

// 从对象数组中抽取指定列，组成新的数组返回
export function getElemsFromArr(elemName, arr, i18n = false, prefix = '') {
    let elems = []
    if (Array.isArray(arr) && arr.length > 0) {
        for (let index = 0; index < arr.length; index++) {
            const element = arr[index]
            if (element[elemName]) {
                elems.push(
                    i18n ? window.vm.$t(prefix + element[elemName]) : element[elemName]
                )
            }
        }
    }
    return elems
}

// 从对象数组中抽取指定列，组成新的对象数组返回
export function getElemsObjFromArr(elemName, arr, i18n = false, prefix = '') {
    let elems = []
    if (Array.isArray(arr) && arr.length > 0) {
        for (let index = 0; index < arr.length; index++) {
            const element = arr[index]
            if (element[elemName]) {
                // elems.push(
                //   i18n ? window.vm.$t(prefix + element[elemName]) : element[elemName]
                // )
                var opt = {}
                opt.key = element.key
                opt.label = i18n
                    ? window.vm.$t(prefix + element[elemName])
                    : element[elemName]
                elems.push(opt)
            }
        }
    }
    return elems
}

// 字数超长后显示省略号
export function getEllipsis(value) {
    if (value.length > 10) {
        return value.substr(0, 10) + '...'
    }
    return value
}

// 验证手机号
export function verifyPhone(phone) {
    return /^1[34578]\d{9}$/.test(phone)
}

// 下载文档
export function downFile(url, filename) {
    var oReq = new XMLHttpRequest()

    oReq.open('GET', url, true)
    oReq.setRequestHeader('Authorization', getToken())
    oReq.responseType = 'blob'
    oReq.onload = function (oEvent) {
        var content = oReq.response
        var elink = document.createElement('a')
        elink.download = filename
        elink.style.display = 'none'
        var blob = new Blob([content])
        elink.href = URL.createObjectURL(blob)
        document.body.appendChild(elink)
        elink.click()
        document.body.removeChild(elink)
    }
    oReq.send()
}

// 判断是否IE的Edge浏览器
export function IsIE() {
    var userAgent = navigator.userAgent //取得浏览器的userAgent字符串
    var isIE =
        userAgent.indexOf('compatible') > -1 && userAgent.indexOf('MSIE') > -1 //判断是否IE<11浏览器
    var isEdge = userAgent.indexOf('Edge') > -1 && !isIE //判断是否IE的Edge浏览器
    var isIE11 =
        userAgent.indexOf('Trident') > -1 && userAgent.indexOf('rv:11.0') > -1
    if (isIE) {
        return true
    } else if (isEdge) {
        return false
    } else if (isIE11) {
        return true
    } else {
        return false //不是ie浏览器
    }
}

/**
 * 获取表格的可视化高度：针对措施追踪、总结归档
 * @param dom DOM元素
 * @param fromH 表单高度
 */
export function getTableHeight(dom, fromH) {
    // 页面顶部高度
    let antLayoutHeaderH = document.querySelector(".ant-layout-header").offsetHeight;
    // 页面容器margin：选项卡顶部高度 - 页面顶部高度
    let antTabsDom = document.querySelector(".ant-card");
    let antLayoutContentMargin = antTabsDom.offsetTop - antLayoutHeaderH;
    // 选项卡border
    let antCardBorderedStyle = window.getComputedStyle(document.querySelector(".ant-card-bordered"), null);
    let antCardBorderedBorder = parseFloat(antCardBorderedStyle.getPropertyValue('border-top'));
    // 选项卡padding
    let antCardBodyStyle = window.getComputedStyle(document.querySelector(".ant-card-body"), null);
    let antCardBodyPadding = parseFloat(antCardBodyStyle.getPropertyValue('padding'));
    // 选项卡高度
    let antTabsBarH = document.querySelector(".ant-tabs-bar").offsetHeight;
    // 选项卡margin-bottom
    let antTabsBarStyle = window.getComputedStyle(document.querySelector(".ant-tabs-bar"), null);
    let antTabsBarStyleMarginBottom = parseFloat(antTabsBarStyle.getPropertyValue('margin-bottom'));
    // 表格表头高度
    let antTableTheadH = dom.querySelector(".ant-table-thead").offsetHeight;
    // 表格border
    let antTableSmallStyle = window.getComputedStyle(dom.querySelector(".ant-table-small"), null);
    let antTableSmallBorder = parseFloat(antTableSmallStyle.getPropertyValue('border-top'));
    // 表格分页56，固定值
    let antTablePaginationH = 56;
    // console.log("页面顶部高度:" + antLayoutHeaderH);
    // console.log("页面容器margin:" + antLayoutContentMargin);
    // console.log("选项卡padding:" + antCardBodyPadding);
    // console.log("选项卡border:" + antCardBorderedBorder);
    // console.log("选项卡高度:" + antTabsBarH);
    // console.log("选项卡margin-bottom:" + antTabsBarStyleMarginBottom);
    // console.log("表单:" + fromH);
    // console.log("表格表头高度:" + antTableTheadH);
    // console.log("表格border:" + antTableSmallBorder);
    // console.log("表格：分页:" + antTablePaginationH);

    let otherDomH = antLayoutHeaderH + (antLayoutContentMargin * 2) + (antCardBodyPadding * 2)
        + (antCardBorderedBorder * 2) + antTabsBarH + antTabsBarStyleMarginBottom + fromH
        + antTableTheadH + (antTableSmallBorder * 2) + antTablePaginationH;
    console.log("其他元素高度:" + otherDomH);
    return otherDomH;
}
